@@ -185,10 +185,13 @@ if not CACHE['INITIALIZED']:
                     widget_can_view = get_metadata(attr, 'can_view')
                     widget_dashboard = get_metadata(attr, 'dashboard')
                     widget_formatter = get_metadata(attr, 'formatter')
+                    widget_shortcut = get_metadata(attr, 'shortcut')
                     widget_icon = get_metadata(attr, 'icon')
+                    widget_url = '{}{}/'.format(url, attr.__func__.__name__)
                     widget = dict(
                         verbose_name=widget_verbose_name, model=model, function=attr_name, can_view=widget_can_view,
-                        dashboard=widget_dashboard, formatter=widget_formatter, link=False, icon=widget_icon
+                        dashboard=widget_dashboard, formatter=widget_formatter, link=False, icon=widget_icon,
+                        shortcut=widget_shortcut, url=widget_url
                     )
                     CACHE['SUBSET_WIDGETS'].append(widget)
                     if model not in CACHE['MANAGER_METHODS']:
