@@ -67,7 +67,9 @@ def listt(request, app, cls, subset=None):
 
     qs = _model.objects.all(request.user)
     if subset:
-        qs = getattr(qs, subset)()
+        subset_func = getattr(qs, subset)
+        parameters = get_role_values_for_condition(subset_func, request.user)
+        qs = subset_func(*parameters)
     list_subsets = subset and [subset] or None
 
     paginator = Paginator(request, qs, title, list_subsets=list_subsets, is_list_view=True,
