@@ -1,33 +1,34 @@
 # -*- coding: utf-8 -*-
 
-import json
 import six
-import sys
-from django.core.exceptions import ValidationError
+import operator
 from django.db.models import base
 from django.db.models import Q
-from django.conf import settings
 from django.db.models import query
-from operator import __or__ as OR
 from djangoplus.db.models.fields import *
 import django.db.models.options as options
 from django.db.models.aggregates import Sum, Avg
 from django.db.models.deletion import Collector
-from djangoplus.utils.metadata import get_metadata, getattr2, find_model, get_field, check_role
+
+from djangoplus.utils import list_one_to_one_field_names, list_field_names, format_value
+from djangoplus.utils.metadata import get_metadata, find_model, get_field, check_role
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.auth import get_user_model
 from functools import reduce
 
 setattr(options, 'DEFAULT_NAMES', options.DEFAULT_NAMES + (
     'icon', 'verbose_female', 'order_by', 'pdf', 'menu',
-    'list_display', 'list_per_page', 'list_template', 'list_total', 'list_shortcut', 'list_csv', 'list_xls', 'list_menu', 'list_lookups', 'list_pdf',
-    'add_label', 'add_form', 'add_inline', 'add_message', 'add_shortcut', 'select_template', 'select_display', 'select_related',
-    'log', 'logging',
+    'list_display', 'list_per_page', 'list_template', 'list_total', 'list_shortcut',
+    'list_csv', 'list_xls', 'list_menu', 'list_lookups', 'list_pdf',
+    'add_label', 'add_form', 'add_message', 'add_shortcut', 'add_style',
+    'select_template', 'select_display', 'select_related', 'log', 'logging',
     'can_add', 'can_edit', 'can_delete', 'can_list', 'can_view', 'can_admin',
     'can_list_by_role', 'can_view_by_role', 'can_add_by_role', 'can_edit_by_role', 'can_admin_by_role',
     'can_list_by_unit', 'can_view_by_unit', 'can_add_by_unit', 'can_edit_by_unit', 'can_admin_by_unit',
-    'can_list_by_organization', 'can_view_by_organization', 'can_add_by_organization', 'can_edit_by_organization', 'can_admin_by_organization',
-    'usecase', 'class_diagram',
+    'can_list_by_organization', 'can_view_by_organization', 'can_add_by_organization',
+    'can_edit_by_organization', 'can_admin_by_organization',
+    'usecase', 'class_diagram', 'dashboard',
+    'expose', 'expose_fields'
 ))
 
 
@@ -82,15 +83,15 @@ class QueryStatistics(object):
     def as_table(self, request=None):
         from djangoplus.ui.components.utils import QueryStatisticsTable
         self._totalize()
-        return QueryStatisticsTable(request, self)
+        return QueryStatisticsTable(self, request=request)
 
     def as_chart(self, request=None):
         from djangoplus.ui.components.utils import QueryStatisticsChart
         if not self.groups:
-            chart = QueryStatisticsChart(request, self)
+            chart = QueryStatisticsChart(self, request=request)
             return chart.donut()
         else:
-            chart = QueryStatisticsChart(request, self)
+            chart = QueryStatisticsChart(self, request=request)
             if self.labels and self.labels[0] == 'Jan':
                 return chart.line()
             else:
@@ -142,7 +143,10 @@ class QuerySet(query.QuerySet):
     def __sub__(self, other):
         return self.difference(other)
 
-    def all(self, user=None, obj=None):
+    def all(self, user=None):
+        return self.contextualize(user)
+
+    def contextualize(self, user, obj=None):
         app_label = get_metadata(self.model, 'app_label')
         if user:
             role_username = get_metadata(user, 'role_username')
@@ -151,19 +155,24 @@ class QuerySet(query.QuerySet):
         queryset = self._clone()
         queryset._user = user
         if user:
-            self_permission = '{}.view_{}'.format(app_label, self.model.__name__.lower())
-            obj_permission = obj and '{}.view_{}'.format(get_metadata(type(obj), 'app_label'), type(obj).__name__.lower())
+            self_permission = '{}.view_{}'.format(
+                app_label, self.model.__name__.lower()
+            )
+            obj_permission = obj and '{}.view_{}'.format(
+                get_metadata(type(obj), 'app_label'), type(obj).__name__.lower()
+            )
             has_perm = obj_permission and user.has_perm(obj_permission) or user.has_perm(self_permission)
         else:
             has_perm = True
         if has_perm:
             if user and not user.is_superuser:
+                queryset = user.apply_current_scope(queryset)
                 permission_mapping = user.get_permission_mapping(self.model, obj)
                 if 'list_lookups' in permission_mapping and permission_mapping['list_lookups']:
-                    l = []
+                    lookups = []
                     for lookup, value in permission_mapping['list_lookups']:
-                        l.append(Q(**{'{}__in'.format(lookup): value}))
-                    return queryset.filter(reduce(OR, l))
+                        lookups.append(Q(**{'{}__in'.format(lookup): value}))
+                    return queryset.filter(reduce(operator.__or__, lookups)).distinct()
             return queryset
         return self.none()
      
@@ -188,7 +197,8 @@ class QuerySet(query.QuerySet):
 
             if horizontal_key:
                 iterator_model = get_field(self.model, horizontal_key).remote_field.model
-                iterators = iterator_model.objects.filter(pk__in=self.values_list(horizontal_key, flat=True).order_by(horizontal_key).distinct())
+                iterators = iterator_model.objects.filter(pk__in=self.values_list(
+                    horizontal_key, flat=True).order_by(horizontal_key).distinct())
                 horizontal_field = get_field(self.model, horizontal_key)
                 title = '{} anual por {}'.format(verbose_name, horizontal_field.verbose_name)
                 statistics = QueryStatistics(title)
@@ -196,7 +206,9 @@ class QuerySet(query.QuerySet):
                     group = str(iterator)
                     for i, month in enumerate(months):
                         label = month
-                        qs = self.filter(**{'{}__month'.format(vertical_key): i+1, horizontal_key: iterator.pk})
+                        qs = self.filter(
+                            **{'{}__month'.format(vertical_key): i+1, horizontal_key: iterator.pk}
+                        )
                         statistics.add(label, qs, qs.count(), group)
                 return statistics
             else:
@@ -234,11 +246,18 @@ class QuerySet(query.QuerySet):
                 vertical_choices = [(True, 'Sim'), (False, 'Não')]
             else:
                 vertical_model = find_model(self.model, vertical_key)
-                vertical_choices = [(o.pk, str(o)) for o in vertical_model.objects.filter(id__in=self.values_list(vertical_key, flat=True))]
+                if vertical_model == self.model:
+                    vertical_choices = [(getattr(o, vertical_key), str(o)) for o in vertical_model.objects.all()]
+                else:
+                    vertical_choices = [(o.pk, str(o)) for o in vertical_model.objects.filter(
+                        id__in=self.values_list(vertical_key, flat=True))]
             if horizontal_key:
+                horizontal_choices = []
                 horizontal_field = get_field(self.model, horizontal_key)
                 if horizontal_field.choices:
-                    used_choices = self.values_list(horizontal_key, flat=True).order_by(horizontal_key).distinct()
+                    used_choices = self.values_list(
+                        horizontal_key, flat=True
+                    ).order_by(horizontal_key).distinct()
                     horizontal_choices = []
                     for choice in horizontal_field.choices:
                         if choice[0] in used_choices:
@@ -247,9 +266,12 @@ class QuerySet(query.QuerySet):
                     vertical_choices = [(True, 'Sim'), (False, 'Não')]
                 else:
                     horizontal_model = find_model(self.model, horizontal_key)
-                    horizontal_choices = [(o.pk, str(o)) for o in horizontal_model.objects.filter(id__in=self.values_list(horizontal_key, flat=True))]
+                    horizontal_choices = [(o.pk, str(o)) for o in horizontal_model.objects.filter(
+                        id__in=self.values_list(horizontal_key, flat=True))]
 
-                title = '{} por {} e {}'.format(verbose_name, vertical_field.verbose_name.lower(), horizontal_field.verbose_name)
+                title = '{} por {} e {}'.format(
+                    verbose_name, vertical_field.verbose_name.lower(), horizontal_field.verbose_name
+                )
                 statistics = QueryStatistics(title)
                 for vertical_choice in vertical_choices:
                     group = vertical_choice[1]
@@ -257,8 +279,11 @@ class QuerySet(query.QuerySet):
                     for horizontal_choice in horizontal_choices:
                         label = horizontal_choice[1]
                         value = 0
-                        lookup = {vertical_key: vertical_choice[0] or None, horizontal_key: horizontal_choice[0] or None}
-                        qs = self.filter(**lookup)
+                        lookup = {
+                            vertical_key: vertical_choice[0] or None,
+                            horizontal_key: horizontal_choice[0] or None
+                        }
+                        qs = self.filter(**lookup).distinct()
                         if aggregate:
                             mode, attr = aggregate
                             if mode == 'sum':
@@ -281,7 +306,7 @@ class QuerySet(query.QuerySet):
                     label = vertical_choice[1]
                     lookup = {vertical_key: vertical_choice[0] or None}
                     value = 0
-                    qs = self.filter(**lookup)
+                    qs = self.filter(**lookup).distinct()
                     if aggregate:
                         mode, attr = aggregate
                         if mode == 'sum':
@@ -324,8 +349,10 @@ class Manager(models.Manager):
     def get_queryset(self):
         return self.queryset_class(self.model, using=self._db)
 
-    def all(self, user=None, obj=None):
-        return self.get_queryset().all(user, obj=obj)
+    def all(self, user=None):
+        if self.queryset_class != self.model.objects.queryset_class:
+            return self.model.objects.filter(pk=0)|self.get_queryset().all(user)
+        return self.get_queryset().all(user)
 
     def count(self, vertical_key=None, horizontal_key=None):
         return self.get_queryset().count(vertical_key, horizontal_key)
@@ -333,11 +360,23 @@ class Manager(models.Manager):
     def sum(self, attr, vertical_key=None, horizontal_key=None):
         return self.get_queryset().sum(attr, vertical_key, horizontal_key)
 
+    def authenticated(self, obj):
+        lookup = get_metadata(self.model, 'role_username')
+        qs = self.model.objects.filter(**{lookup: obj._user.username})
+        if qs.count():
+            if qs.count() == 1:
+                return qs[0]
+            else:
+                return qs
+        else:
+            return None
+
 
 class ModelBase(base.ModelBase):
     def __new__(mcs, name, bases, attrs):
         meta_new = super(ModelBase, mcs).__new__
-        module = __import__(attrs['__module__'], fromlist=list(map(str, attrs['__module__'].split('.'))))
+        fromlist = list(map(str, attrs['__module__'].split('.')))
+        module = __import__(attrs['__module__'], fromlist=fromlist)
         if hasattr(module, '{}Manager'.format(name)):
             queryset_class = getattr(module, '{}Manager'.format(name))
             if issubclass(queryset_class, QuerySet):
@@ -367,6 +406,7 @@ class ModelBase(base.ModelBase):
 
         return cls
 
+
 DefaultManager = QuerySet
 
 
@@ -388,7 +428,9 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
                 subinstance = getattr(self, subclass.__name__.lower())
                 if hasattr(subinstance, '__str__'):
                     return subinstance.__str__()
-        return '{}{}'.format(get_metadata(self.__class__, 'verbose_name'), self.pk and ' #{}'.format(self.pk or ''))
+        return '{}{}'.format(
+            get_metadata(self.__class__, 'verbose_name'), self.pk and ' #{}'.format(self.pk or '')
+        )
 
     @classmethod
     def update_metadata(cls, **kwargs):
@@ -422,16 +464,23 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
 
     def get_children(self):
         tree_index = getattr(self, self.get_tree_index_field().name)
-        return type(self).objects.filter(**{'{}__startswith'.format(tree_index):tree_index}).exclude(pk=self.pk)
+        return type(self).objects.filter(
+            **{'{}__startswith'.format(tree_index): tree_index}
+        ).exclude(pk=self.pk)
 
     def save(self, *args, **kwargs):
         log_data = get_metadata(self.__class__, 'log', False)
         log_index = get_metadata(self.__class__, 'logging', (), iterable=True)
 
+        log_index = [index for index in log_index]
+        if log_data or settings.LOG_MODELS:
+            for field_name in list_one_to_one_field_names(type(self)):
+                log_index.append(field_name)
+
         # edited changes logging
-        if (log_data or log_index) and self.pk and self._user:
+        if (log_data or log_index or settings.LOG_MODELS) and self.pk and self._user:
             log_indexes = list(log_index)
-            qs = self.__class__.objects.filter(pk=self.pk).select_related(*log_indexes)
+            qs = type(self).objects.filter(pk=self.pk).select_related(*log_indexes)
 
             self._diff = False
             if qs.exists():
@@ -445,18 +494,19 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
                 log.object_description = str(self)
                 diff = []
 
-                for field in get_metadata(self, 'fields'):
+                for field in get_metadata(type(self), 'fields'):
                     o1 = getattr(old, field.name)
                     o2 = getattr(self, field.name)
-                    v1 = str(o1)
-                    v2 = str(o2)
-                    if not hasattr(o2, '__iter__') and v1 != v2:
+                    v1 = format_value(o1)
+                    v2 = format_value(o2)
+
+                    if v1 != v2:
                         self._diff = True
                         diff.append((field.verbose_name, v1, v2))
-
-                log.content = json.dumps(diff)
-                log.save()
-                log.create_indexes(old)
+                if diff or log_index:
+                    log.content = json.dumps(diff)
+                    log.save()
+                    log.create_indexes(old)
 
         # tree model
         tree_index_field = self.get_tree_index_field()
@@ -500,7 +550,7 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
                 obj.save(*args, **kwargs)
 
         # registration logging
-        if (log_data or log_index) and not hasattr(self, '_diff') and self._user:
+        if (log_data or log_index or settings.LOG_MODELS) and not hasattr(self, '_diff') and self._user:
             from djangoplus.admin.models import Log
             log = Log()
             log.operation = Log.ADD
@@ -524,7 +574,7 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
         log_data = get_metadata(self.__class__, 'log', False)
         log_index = get_metadata(self.__class__, 'logging', ())
 
-        if (log_data or log_index) and self._user:
+        if (log_data or log_index or settings.LOG_MODELS) and self._user:
             from djangoplus.admin.models import Log
 
             collector = Collector(using='default')
@@ -541,9 +591,12 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
                     diff = []
                     for field in get_metadata(obj.__class__, 'fields'):
                         if not isinstance(field, models.FileField):
-                            o1 = getattr(obj, field.name)
-                            v1 = str(o1)
-                            diff.append((field.verbose_name, v1))
+                            try:
+                                o1 = getattr(obj, field.name)
+                                v1 = format_value(o1)
+                                diff.append((field.verbose_name, v1, None))
+                            except Exception:
+                                pass
 
                     log.content = json.dumps(diff)
                     log.save()
@@ -570,10 +623,10 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
                     permission_mapping = self._user.get_permission_mapping(model)
                     if 'edit_lookups' in permission_mapping and permission_mapping['edit_lookups']:
                         for lookup, values in permission_mapping['edit_lookups']:
-                            value = getattr2(self, lookup)
-                            value = hasattr(value, 'pk') and value.pk or value
-                            if value in values:
-                                return True
+                            qs = model.objects.filter(pk=self.pk)
+                            for value in qs.values_list(lookup, flat=True).distinct():
+                                if value in values:
+                                    return True
                         return False
                 else:
                     return False
@@ -589,10 +642,10 @@ class Model(six.with_metaclass(ModelBase, models.Model)):
                     permission_mapping = self._user.get_permission_mapping(model)
                     if 'delete_lookups' in permission_mapping and permission_mapping['delete_lookups']:
                         for lookup, values in permission_mapping['delete_lookups']:
-                            value = getattr2(self, lookup)
-                            value = hasattr(value, 'pk') and value.pk or value
-                            if value in values:
-                                return True
+                            qs = model.objects.filter(pk=self.pk)
+                            for value in qs.values_list(lookup, flat=True).distinct():
+                                if value in values:
+                                    return True
                         return False
                 else:
                     return False
