@@ -348,14 +348,17 @@ class QueryStatisticsTable(Table):
             rows = []
             footer = []
             if queryset_statistics.series:
+                add_xtotal = False
                 for i, group in enumerate(queryset_statistics.groups):
                     row = [group]
                     for value in queryset_statistics.series[i]:
                         row.append(value)
                     if len(queryset_statistics.series) > 1:
-                        row.append(queryset_statistics.xtotal[i])
+                        if i < len(queryset_statistics.xtotal):
+                            row.append(queryset_statistics.xtotal[i])
+                            add_xtotal = True
                     rows.append(row)
-                if len(queryset_statistics.series) > 1:
+                if add_xtotal:
                     footer = [' '] + queryset_statistics.ytotal + [queryset_statistics.total()]
                     header = [' '] + queryset_statistics.labels + [' ']
                 else:
